-------------------------------------------------------------------------------
--! @file       CipherCore.vhd
--! @brief      Cipher core template module
--! @project    CAESAR Candidate Evaluation
--! @author     Ahmed Ferozpuri
--! @copyright  Copyright (c) 2016 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             -unrestricted)
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.PRIMATEsPKG.ALL;
use work.AEAD_pkg.all;

entity CipherCore is
    generic (
        --! Reset behavior
        G_ASYNC_RSTN    : boolean := False; --! Async active low reset
        --! Block size (bits)
        G_DBLK_SIZE     : integer := 40;   --! Data
        G_KEY_SIZE      : integer := 120;   --! Key
        G_TAG_SIZE      : integer := 120;   --! Tag
        --! The number of bits required to hold block size expressed in
        --! bytes = log2_ceil(G_DBLK_SIZE/8)
        G_LBS_BYTES     : integer := 4;
		--! Maximum supported AD/message/ciphertext length = 2^G_MAX_LEN-1
        G_MAX_LEN       : integer := SINGLE_PASS_MAX
    );
    port (
        --! Global
        clk             : in  std_logic;
        rst             : in  std_logic;
        --! PreProcessor (data)
        key             : in  std_logic_vector(G_KEY_SIZE       -1 downto 0);
        bdi             : in  std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        --! PreProcessor (controls)
        key_ready       : out std_logic;
        key_valid       : in  std_logic;
        key_update      : in  std_logic;
        decrypt         : in  std_logic;
        bdi_ready       : out std_logic;
        bdi_valid       : in  std_logic;
        bdi_type        : in  std_logic_vector(3                -1 downto 0);
        bdi_partial     : in  std_logic;
        bdi_eot         : in  std_logic;
        bdi_eoi         : in  std_logic;
        bdi_size        : in  std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        bdi_valid_bytes : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        bdi_pad_loc     : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        --! PostProcessor
        bdo             : out std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        bdo_valid       : out std_logic;
        bdo_ready       : in  std_logic;
        bdo_size        : out std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        msg_auth_done   : out std_logic;
        msg_auth_valid  : out std_logic
    );
end entity CipherCore;

ARCHITECTURE structure OF CipherCore IS

	CONSTANT W : INTEGER := G_DBLK_SIZE;
	CONSTANT S : INTEGER := G_KEY_SIZE;
	CONSTANT Z : INTEGER := G_KEY_SIZE;
	
	SIGNAL init, run, mode, donel, selBDO, ldTag, enTag, lastBlockFull: std_logic;
	SIGNAL pn : std_logic_vector(1 DOWNTO 0);
	SIGNAL selV : std_logic_vector(1 DOWNTO 0);
	SIGNAL Yl : std_logic_vector(W-1 DOWNTO 0);
	
	SIGNAL npub_en, etag_en, tag_en, tag_ld : std_logic;
	SIGNAL bdo_sel : std_logic;
	
BEGIN
	
	HANUMAN: ENTITY work.HANUMAN(behavioral)
	GENERIC MAP (S => S, V => 2*S+40)
	PORT MAP (clk => clk, init => init, run => run, mode => mode, selV => selV(1 DOWNTO 0),
				 lastBlockFull => lastBlockFull, pn => pn, key_ready => key_valid,
				 bdi => bdi, key => key, bdi_pad_loc => bdi_pad_loc,
				 bdo => bdo, bdo_sel => bdo_sel,
				 npub_en => npub_en, etag_en => etag_en, tag_en => tag_en, tag_ld => tag_ld,
				 done => donel, msg_auth_valid => msg_auth_valid);

	PCCTRL: ENTITY work.PCCTRL(behavioral)
	PORT MAP (clk => clk, rst => rst, bdi_ready => bdi_ready, bdi_valid => bdi_valid, key_valid => key_valid, key_ready => key_ready, bdo_ready => bdo_ready, bdi_type => bdi_type,
				 done => donel, bdi_eoi => bdi_eoi, bdi_eot => bdi_eot, bdi_size => bdi_size, msg_auth_done => msg_auth_done,
				 init => init, run => run, lastBlockFull => lastBlockFull, decrypt => decrypt, mode => mode,
				 npub_en => npub_en, etag_en => etag_en, tag_en => tag_en, tag_ld => tag_ld,
				 bdo_sel => bdo_sel, bdo_valid =>bdo_valid, key_update => key_update, pn => pn, selV => selV);
	
END structure;